@@ -1,10 +1,5 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import print_function
-
 import sys
 import os
-import codecs
 import io
 from errno import EPIPE
 
@@ -12,19 +7,11 @@ from . import rbql_engine
 from . import csv_utils
 
 
-PY3 = sys.version_info[0] == 3
-polymorphic_xrange = range if PY3 else xrange
-
 default_csv_encoding = 'utf-8'
 ansi_reset_color_code = '\u001b[0m'
 
 debug_mode = False
 
-try:
-    broken_pipe_exception = BrokenPipeError
-except NameError: # Python 2
-    broken_pipe_exception = IOError
-
 
 def is_ascii(s):
     return all(ord(c) < 128 for c in s)
@@ -58,31 +45,23 @@ def interpret_named_csv_format(format_name):
 def encode_input_stream(stream, encoding):
     if encoding is None:
         return stream
-    if PY3:
-        # Reference: https://stackoverflow.com/a/16549381/2898283
-        # typical stream (e.g. sys.stdin) in Python 3 is actually a io.TextIOWrapper but with some unknown encoding
-        try:
-            return io.TextIOWrapper(stream.buffer, encoding=encoding)
-        except AttributeError:
-            # BytesIO doesn't have "buffer"
-            return io.TextIOWrapper(stream, encoding=encoding)
-    else:
-        # Reference: https://stackoverflow.com/a/27425797/2898283
-        # Python 2 streams don't have stream.buffer and therefore we can't use io.TextIOWrapper. Instead we use codecs
-        return codecs.getreader(encoding)(stream)
+    # Reference: https://stackoverflow.com/a/16549381/2898283
+    # typical stream (e.g. sys.stdin) in Python 3 is actually a io.TextIOWrapper but with some unknown encoding
+    try:
+        return io.TextIOWrapper(stream.buffer, encoding=encoding)
+    except AttributeError:
+        # BytesIO doesn't have "buffer"
+        return io.TextIOWrapper(stream, encoding=encoding)
 
 
 def encode_output_stream(stream, encoding):
     if encoding is None:
         return stream
-    if PY3:
-        try:
-            return io.TextIOWrapper(stream.buffer, encoding=encoding)
-        except AttributeError:
-            # BytesIO doesn't have "buffer"
-            return io.TextIOWrapper(stream, encoding=encoding)
-    else:
-        return codecs.getwriter(encoding)(stream)
+    try:
+        return io.TextIOWrapper(stream.buffer, encoding=encoding)
+    except AttributeError:
+        # BytesIO doesn't have "buffer"
+        return io.TextIOWrapper(stream, encoding=encoding)
 
 
 def remove_utf8_bom(line, assumed_source_encoding):
@@ -246,8 +225,8 @@ class CSVWriter(rbql_engine.RBQLOutputWriter):
                 self.stream.write(ansi_reset_color_code)
             self.stream.write(self.line_separator)
             return True
-        except broken_pipe_exception as exc:
-            if broken_pipe_exception == IOError:
+        except BrokenPipeError as exc:
+            if BrokenPipeError == IOError: # FIXME perhaps we don't need this? Maybe it was needed for Python 2 only?
                 if exc.errno != EPIPE:
                     raise
             self.broken_pipe = True
@@ -255,17 +234,17 @@ class CSVWriter(rbql_engine.RBQLOutputWriter):
 
 
     def colorize_fields(self, fields):
-        for i in polymorphic_xrange(len(fields)):
+        for i in range(len(fields)):
             fields[i] = self.colors[i % len(self.colors)] + fields[i]
 
 
     def quote_fields(self, fields):
-        for i in polymorphic_xrange(len(fields)):
+        for i in range(len(fields)):
             fields[i] = csv_utils.quote_field(fields[i], self.delim)
 
 
     def quote_fields_rfc(self, fields):
-        for i in polymorphic_xrange(len(fields)):
+        for i in range(len(fields)):
             fields[i] = csv_utils.rfc_quote_field(fields[i], self.delim)
 
 
@@ -275,10 +254,8 @@ class CSVWriter(rbql_engine.RBQLOutputWriter):
 
 
     def normalize_fields(self, fields):
-        for i in polymorphic_xrange(len(fields)):
-            if PY3 and isinstance(fields[i], str):
-                continue
-            elif not PY3 and isinstance(fields[i], basestring):
+        for i in range(len(fields)):
+            if isinstance(fields[i], str):
                 continue
             elif fields[i] is None:
                 fields[i] = ''
@@ -306,8 +283,8 @@ class CSVWriter(rbql_engine.RBQLOutputWriter):
                 self.stream.flush() # This flush still can throw if all flushes before were sucessfull! And the exceptions would be printed anyway, even if it was explicitly catched just couple of lines after.
                 # Basically this fails if output is small and this is the first flush after the pipe was broken e.g. second flush if piped to head -n 1
                 # Here head -n 1 finished after the first flush, and the final explict flush here just killing it
-            except broken_pipe_exception as exc:
-                if broken_pipe_exception == IOError:
+            except BrokenPipeError as exc:
+                if BrokenPipeError == IOError: # FIXME perhaps we don't need this? Maybe it was needed for Python 2 only?
                     if exc.errno != EPIPE:
                         raise
                 # In order to avoid BrokenPipeError from being printed as a warning to stderr, we need to perform this magic below. See:
@@ -329,7 +306,7 @@ class CSVWriter(rbql_engine.RBQLOutputWriter):
 
 
 class CSVRecordIterator(rbql_engine.RBQLInputIterator):
-    def __init__(self, stream, encoding, delim, policy, has_header=False, comment_prefix=None, table_name='input', variable_prefix='a', chunk_size=1024, line_mode=False):
+    def __init__(self, stream, encoding, delim, policy, has_header=False, comment_prefix=None, table_name='input', variable_prefix='a', chunk_size=1024, line_mode=False, strip_whitespaces=False):
         assert encoding in ['utf-8', 'latin-1', None]
         self.encoding = encoding
         self.stream = encode_input_stream(stream, encoding)
@@ -338,6 +315,7 @@ class CSVRecordIterator(rbql_engine.RBQLInputIterator):
         self.table_name = table_name
         self.variable_prefix = variable_prefix
         self.comment_prefix = comment_prefix if (comment_prefix is not None and len(comment_prefix)) else None
+        self.strip_whitespaces = strip_whitespaces
 
         self.buffer = ''
         self.detected_line_separator = '\n'
@@ -464,6 +442,8 @@ class CSVRecordIterator(rbql_engine.RBQLInputIterator):
                 break
         self.NR += 1
         record, warning = csv_utils.smart_split(line, self.delim, self.policy, preserve_quotes_and_whitespaces=False)
+        if self.strip_whitespaces:
+            record = [v.strip() for v in record]
         if warning:
             if self.first_defective_line is None:
                 self.first_defective_line = self.NL
@@ -509,7 +489,7 @@ class CSVRecordIterator(rbql_engine.RBQLInputIterator):
 
 
 class FileSystemCSVRegistry(rbql_engine.RBQLTableRegistry):
-    def __init__(self, input_file_dir, delim, policy, encoding, has_header, comment_prefix):
+    def __init__(self, input_file_dir, delim, policy, encoding, has_header, comment_prefix, strip_whitespaces):
         self.input_file_dir = input_file_dir
         self.delim = delim
         self.policy = policy
@@ -518,6 +498,7 @@ class FileSystemCSVRegistry(rbql_engine.RBQLTableRegistry):
         self.input_stream = None
         self.has_header = has_header
         self.comment_prefix = comment_prefix
+        self.strip_whitespaces = strip_whitespaces
         self.table_path = None
 
     def get_iterator_by_table_id(self, table_id, single_char_alias):
@@ -525,7 +506,7 @@ class FileSystemCSVRegistry(rbql_engine.RBQLTableRegistry):
         if self.table_path is None:
             raise rbql_engine.RbqlIOHandlingError('Unable to find join table "{}"'.format(table_id))
         self.input_stream = open(self.table_path, 'rb')
-        self.record_iterator = CSVRecordIterator(self.input_stream, self.encoding, self.delim, self.policy, self.has_header, comment_prefix=self.comment_prefix, table_name=table_id, variable_prefix=single_char_alias)
+        self.record_iterator = CSVRecordIterator(self.input_stream, self.encoding, self.delim, self.policy, self.has_header, comment_prefix=self.comment_prefix, table_name=table_id, variable_prefix=single_char_alias, strip_whitespaces=self.strip_whitespaces)
         return self.record_iterator
 
     def finish(self):
@@ -539,7 +520,7 @@ class FileSystemCSVRegistry(rbql_engine.RBQLTableRegistry):
         return result
 
 
-def query_csv(query_text, input_path, input_delim, input_policy, output_path, output_delim, output_policy, csv_encoding, output_warnings, with_headers, comment_prefix=None, user_init_code='', colorize_output=False):
+def query_csv(query_text, input_path, input_delim, input_policy, output_path, output_delim, output_policy, csv_encoding, output_warnings, with_headers, comment_prefix=None, user_init_code='', colorize_output=False, strip_whitespaces=False):
     output_stream, close_output_on_finish = (None, False)
     input_stream, close_input_on_finish = (None, False)
     join_tables_registry = None
@@ -563,8 +544,8 @@ def query_csv(query_text, input_path, input_delim, input_policy, output_path, ou
             user_init_code = read_user_init_code(default_init_source_path)
 
         input_file_dir = None if not input_path else os.path.dirname(input_path)
-        join_tables_registry = FileSystemCSVRegistry(input_file_dir, input_delim, input_policy, csv_encoding, with_headers, comment_prefix)
-        input_iterator = CSVRecordIterator(input_stream, csv_encoding, input_delim, input_policy, with_headers, comment_prefix=comment_prefix)
+        join_tables_registry = FileSystemCSVRegistry(input_file_dir, input_delim, input_policy, csv_encoding, with_headers, comment_prefix, strip_whitespaces)
+        input_iterator = CSVRecordIterator(input_stream, csv_encoding, input_delim, input_policy, with_headers, comment_prefix=comment_prefix, strip_whitespaces=strip_whitespaces)
         output_writer = CSVWriter(output_stream, close_output_on_finish, csv_encoding, output_delim, output_policy, colorize_output=colorize_output)
         if debug_mode:
             rbql_engine.set_debug_mode()
