@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import print_function
-
 import sys
 import re
 import ast
@@ -45,8 +41,6 @@ invalid_keyword_in_aggregate_query_error_msg = '"ORDER BY", "UPDATE" and "DISTIN
 wrong_aggregation_usage_error = 'Usage of RBQL aggregation functions inside Python expressions is not allowed, see the docs'
 numeric_conversion_error = 'Unable to convert value "{}" to int or float. MIN, MAX, SUM, AVG, MEDIAN and VARIANCE aggregate functions convert their string arguments to numeric values'
 
-PY3 = sys.version_info[0] == 3
-
 RBQL_VERSION = __version__
 
 debug_mode = False
@@ -104,10 +98,6 @@ class RBQLContext:
         self.variables_init_code = None
 
 
-def is_str6(val):
-    return (PY3 and isinstance(val, str)) or (not PY3 and isinstance(val, basestring))
-
-
 QueryColumnInfo = namedtuple('QueryColumnInfo', ['table_name', 'column_index', 'column_name', 'is_star', 'alias_name'])
 
 
@@ -162,7 +152,7 @@ def column_info_from_node(root):
         column_name = get_field(root, 'attr')
         if not column_name:
             return None
-        if not is_str6(column_name):
+        if not isinstance(column_name, str):
             return None
         var_root = get_field(root, 'value')
         if not isinstance(var_root, ast.Name):
@@ -181,20 +171,35 @@ def column_info_from_node(root):
         if table_name is None or table_name not in ['a', 'b']:
             return None
         slice_root = get_field(root, 'slice')
-        if slice_root is None or not isinstance(slice_root, ast.Index):
+        if slice_root is None:
             return None
-        slice_val_root = get_field(slice_root, 'value')
         column_index = None
         column_name = None
-        if isinstance(slice_val_root, ast.Str):
-            column_name = get_field(slice_val_root, 's')
-            table_name = None # We don't need table name for named fields
-        elif isinstance(slice_val_root, ast.Num):
-            column_index = get_field(slice_val_root, 'n') - 1
+        if hasattr(ast, 'Index') and isinstance(slice_root, ast.Index):
+            # Important: Since version 3.8 ast.Constant is used instead of ast.Index.
+            # Furthermore ast.Index might be removed from ast in future releases.
+            slice_val_root = get_field(slice_root, 'value')
+            column_index = None
+            column_name = None
+            if isinstance(slice_val_root, ast.Str):
+                column_name = get_field(slice_val_root, 's')
+                table_name = None # We don't need table name for named fields. Updated: But Why???
+            elif isinstance(slice_val_root, ast.Num):
+                column_index = get_field(slice_val_root, 'n') - 1
+            else:
+                return None
+        elif hasattr(ast, 'Constant') and isinstance(slice_root, ast.Constant):
+            # Note: `ast.Constant` replaced `ast.Index` since version 3.8
+            slice_val_root = get_field(slice_root, 'value')
+            if isinstance(slice_val_root, str):
+                column_name = slice_val_root
+                table_name = None # We don't need table name for named fields. Updated: But Why???
+            elif isinstance(slice_val_root, int):
+                column_index = slice_val_root - 1
+            else:
+                return None
         else:
             return None
-        if not PY3 and isinstance(column_name, str):
-            column_name = column_name.decode('utf-8')
         return QueryColumnInfo(table_name=table_name, column_index=column_index, column_name=column_name, is_star=False, alias_name=None)
     column_alias_name = search_for_as_alias_pseudo_function(root)
     if column_alias_name:
@@ -222,12 +227,6 @@ def ast_parse_select_expression_to_column_infos(select_expression):
     return column_infos
 
 
-def iteritems6(x):
-    if PY3:
-        return x.items()
-    return x.iteritems()
-
-
 class RBQLRecord:
     def __init__(self):
         self.storage = dict()
@@ -295,7 +294,7 @@ class NumHandler:
     def parse(self, val):
         if not self.string_detection_done:
             self.string_detection_done = True
-            if is_str6(val):
+            if isinstance(val, str):
                 self.is_str = True
         if not self.is_str:
             return val
@@ -310,6 +309,18 @@ class NumHandler:
             raise RbqlRuntimeError(numeric_conversion_error.format(val)) # UT JSON
 
 
+class AnyValueAggregator:
+    def __init__(self):
+        self.stats = dict()
+
+    def increment(self, key, val):
+        if key not in self.stats:
+            self.stats[key] = val
+
+    def get_final(self, key):
+        return self.stats[key]
+
+
 class MinAggregator:
     def __init__(self):
         self.stats = dict()
@@ -513,7 +524,7 @@ class UniqCountWriter(object):
         return True
 
     def finish(self):
-        for record, cnt in iteritems6(self.records):
+        for record, cnt in self.records.items():
             mutable_record = list(record)
             mutable_record.insert(0, cnt)
             if not self.subwriter.write(mutable_record):
@@ -693,7 +704,7 @@ if not query_context.writer.write(up_fields):
 
 # We need dummy_wrapper_for_exec function because otherwise "import" statements won't work as expected if used inside user-defined functions, see: https://github.com/mechatroner/sublime_rainbow_csv/issues/22
 MAIN_LOOP_BODY = '''
-def dummy_wrapper_for_exec(query_context, user_namespace, LIKE, UNNEST, MIN, MAX, COUNT, SUM, AVG, VARIANCE, MEDIAN, ARRAY_AGG, mad_max, mad_min, mad_sum, select_unnested):
+def dummy_wrapper_for_exec(query_context, user_namespace, LIKE, UNNEST, ANY_VALUE, MIN, MAX, COUNT, SUM, AVG, VARIANCE, MEDIAN, ARRAY_AGG, mad_max, mad_min, mad_sum, select_unnested):
 
     try:
         pass
@@ -704,6 +715,8 @@ def dummy_wrapper_for_exec(query_context, user_namespace, LIKE, UNNEST, MIN, MAX
     like = LIKE
     unnest = UNNEST
     Unnest = UNNEST
+    any_value = ANY_VALUE
+    Any_value = ANY_VALUE
     Min = MIN
     Max = MAX
     count = COUNT
@@ -748,7 +761,7 @@ def dummy_wrapper_for_exec(query_context, user_namespace, LIKE, UNNEST, MIN, MAX
                 raise RbqlParsingError(wrong_aggregation_usage_error) # UT JSON
             raise RbqlRuntimeError('At record ' + str(NR) + ', Details: ' + str(e)) # UT JSON
 
-dummy_wrapper_for_exec(query_context, user_namespace, LIKE, UNNEST, MIN, MAX, COUNT, SUM, AVG, VARIANCE, MEDIAN, ARRAY_AGG, mad_max, mad_min, mad_sum, select_unnested)
+dummy_wrapper_for_exec(query_context, user_namespace, LIKE, UNNEST, ANY_VALUE, MIN, MAX, COUNT, SUM, AVG, VARIANCE, MEDIAN, ARRAY_AGG, mad_max, mad_min, mad_sum, select_unnested)
 '''
 
 
@@ -852,15 +865,15 @@ def compile_and_run(query_context, user_namespace, unit_test_mode=False):
         return res
 
 
+    def ANY_VALUE(val):
+        return init_aggregator(AnyValueAggregator, val) if query_context.aggregation_stage < 2 else val
+
     def MIN(val):
         return init_aggregator(MinAggregator, val) if query_context.aggregation_stage < 2 else val
 
-
-
     def MAX(val):
         return init_aggregator(MaxAggregator, val) if query_context.aggregation_stage < 2 else val
 
-
     def COUNT(_val):
         return init_aggregator(CountAggregator, 1) if query_context.aggregation_stage < 2 else 1
 
@@ -885,9 +898,7 @@ def compile_and_run(query_context, user_namespace, unit_test_mode=False):
     def mad_max(*args, **kwargs):
         single_arg = len(args) == 1 and not kwargs
         if single_arg:
-            if PY3 and isinstance(args[0], str):
-                return MAX(args[0])
-            if not PY3 and isinstance(args[0], basestring):
+            if isinstance(args[0], str):
                 return MAX(args[0])
             if isinstance(args[0], int) or isinstance(args[0], float):
                 return MAX(args[0])
@@ -902,9 +913,7 @@ def compile_and_run(query_context, user_namespace, unit_test_mode=False):
     def mad_min(*args, **kwargs):
         single_arg = len(args) == 1 and not kwargs
         if single_arg:
-            if PY3 and isinstance(args[0], str):
-                return MIN(args[0])
-            if not PY3 and isinstance(args[0], basestring):
+            if isinstance(args[0], str):
                 return MIN(args[0])
             if isinstance(args[0], int) or isinstance(args[0], float):
                 return MIN(args[0])
@@ -1488,6 +1497,7 @@ def shallow_parse_input_query(query_text, input_iterator, tables_registry, query
         if ORDER_BY in rb_actions or UPDATE in rb_actions:
             raise RbqlParsingError(invalid_keyword_in_aggregate_query_error_msg) # UT JSON
         query_context.aggregation_key_expression = '({},)'.format(combine_string_literals(rb_actions[GROUP_BY]['text'], string_literals))
+        query_context.aggregation_stage = 1
 
 
     input_header = input_iterator.get_header()
