@@ -1,6 +1,4 @@
 #!/usr/bin/env python
-from __future__ import unicode_literals
-from __future__ import print_function
 
 import sys
 import os
@@ -21,13 +19,8 @@ from . import _version
 # TODO add an option to align columns for content preview. This would be especially useful for Windows which doesn't support terminal colors
 
 
-PY3 = sys.version_info[0] == 3
-
-
 history_path = os.path.join(os.path.expanduser("~"), ".rbql_py_query_history")
 
-polymorphic_input = input if PY3 else raw_input
-
 
 def eprint(*args, **kwargs):
     print(*args, file=sys.stderr, **kwargs)
@@ -37,9 +30,6 @@ policy_names = ['quoted', 'simple', 'whitespace', 'quoted_rfc', 'monocolumn']
 out_format_names = ['csv', 'tsv', 'input']
 
 
-polymorphic_xrange = range if PY3 else xrange
-
-
 def get_default_policy(delim):
     if delim in [';', ',']:
         return 'quoted'
@@ -87,7 +77,7 @@ def run_with_python_csv(args, is_interactive):
     warnings = []
     error_type, error_msg = None, None
     try:
-        rbql_csv.query_csv(query, input_path, delim, policy, output_path, out_delim, out_policy, csv_encoding, warnings, with_headers, args.comment_prefix, user_init_code, args.color)
+        rbql_csv.query_csv(query, input_path, delim, policy, output_path, out_delim, out_policy, csv_encoding, warnings, with_headers, args.comment_prefix, user_init_code, args.color, strip_whitespaces=args.strip_spaces)
     except Exception as e:
         if args.debug_mode:
             raise
@@ -154,7 +144,7 @@ def sample_lines(src_path, encoding, delim, policy, comment_prefix=None):
     result = []
     with open(src_path, 'rb') as source:
         line_iterator = rbql_csv.CSVRecordIterator(source, encoding, delim=delim, policy=policy, line_mode=True, comment_prefix=comment_prefix)
-        for _i in polymorphic_xrange(10):
+        for _i in range(10):
             line = line_iterator.polymorphic_get_row()
             if line is None:
                 break
@@ -175,9 +165,9 @@ def autodetect_delim_policy(input_path, encoding, comment_prefix=None):
     return (None, None)
 
 
-def sample_records(input_path, delim, policy, encoding, comment_prefix=None):
+def sample_records(input_path, delim, policy, encoding, comment_prefix, strip_whitespaces):
     with open(input_path, 'rb') as source:
-        record_iterator = rbql_csv.CSVRecordIterator(source, encoding, delim=delim, policy=policy, comment_prefix=comment_prefix)
+        record_iterator = rbql_csv.CSVRecordIterator(source, encoding, delim=delim, policy=policy, comment_prefix=comment_prefix, strip_whitespaces=strip_whitespaces)
         sampled_records = record_iterator.get_all_records(num_rows=10);
         warnings = record_iterator.get_warnings()
         return (sampled_records, warnings)
@@ -201,10 +191,7 @@ def print_colorized(records, delim, encoding, show_column_names, with_headers):
                 colored_field = '{}a{}:{}'.format(color_code, i + 1, field)
             out_fields.append(colored_field)
         out_line = delim.join(out_fields) + reset_color_code
-        if PY3:
-            sys.stdout.buffer.write(out_line.encode(encoding))
-        else:
-            sys.stdout.write(out_line.encode(encoding))
+        sys.stdout.buffer.write(out_line.encode(encoding))
         sys.stdout.write('\n')
         sys.stdout.flush()
 
@@ -227,7 +214,7 @@ def run_interactive_loop(mode, args):
         pass
     while True:
         try:
-            query = polymorphic_input('Input SQL-like RBQL query and press Enter:\n> ')
+            query = input('Input SQL-like RBQL query and press Enter:\n> ')
             query = query.strip()
         except EOFError:
             print()
@@ -246,7 +233,8 @@ def run_interactive_loop(mode, args):
         if success:
             print('\nOutput table preview:')
             print('====================================')
-            records, _warnings = sample_records(args.output, args.output_delim, args.output_policy, args.encoding, comment_prefix=None)
+            # Never strip whitespaces in the output preview.
+            records, _warnings = sample_records(args.output, args.output_delim, args.output_policy, args.encoding, comment_prefix=None, strip_whitespaces=False)
             print_colorized(records, args.output_delim, args.encoding, show_column_names=False, with_headers=False)
             print('====================================')
             print('Success! Result table was saved to: ' + args.output)
@@ -277,10 +265,10 @@ def select_table_name_by_user_choice(db_connection):
     else:
         print('Showing database tables:')
     print(', '.join(table_names[:max_to_show]))
-    table_name = polymorphic_input('No input table was provided as a CLI argument, please type in the table name to use:\n> ')
+    table_name = input('No input table was provided as a CLI argument, please type in the table name to use:\n> ')
     table_name = table_name.strip()
     while table_name not in table_names:
-        table_name = polymorphic_input('"{}" is not a valid table name. Please enter a valid table name:\n> '.format(table_name))
+        table_name = input('"{}" is not a valid table name. Please enter a valid table name:\n> '.format(table_name))
         table_name = table_name.strip()
     return table_name
 
@@ -332,7 +320,7 @@ def start_preview_mode_csv(args):
             return
         args.delim = delim
         args.policy = policy
-    records, warnings = sample_records(input_path, delim, policy, args.encoding, args.comment_prefix)
+    records, warnings = sample_records(input_path, delim, policy, args.encoding, args.comment_prefix, args.strip_spaces)
     print('Input table preview:')
     print('====================================')
     print_colorized(records, delim, args.encoding, show_column_names=True, with_headers=args.with_headers)
@@ -387,6 +375,7 @@ def csv_main():
     parser.add_argument('--out-format', help='output format', default='input', choices=out_format_names)
     parser.add_argument('--encoding', help='manually set csv encoding', default=rbql_csv.default_csv_encoding, choices=['latin-1', 'utf-8'])
     parser.add_argument('--output', metavar='FILE', help='write output table to FILE instead of stdout')
+    parser.add_argument('--strip-spaces', action='store_true', help='strip leading and trailing whitespace chars from each input field')
     parser.add_argument('--color', action='store_true', help='colorize columns in output in non-interactive mode')
     parser.add_argument('--version', action='store_true', help='print RBQL version and exit')
     parser.add_argument('--init-source-file', metavar='FILE', help=argparse.SUPPRESS) # Path to init source file to use instead of ~/.rbql_init_source.py
@@ -412,12 +401,6 @@ def csv_main():
         show_error('generic', 'Using "--policy" without "--delim" is not allowed', is_interactive=False)
         sys.exit(1)
 
-    if args.encoding != 'latin-1' and not PY3:
-        if args.delim is not None:
-            args.delim = args.delim.decode(args.encoding)
-        if args.query is not None:
-            args.query = args.query.decode(args.encoding)
-
     is_interactive_mode = args.query is None
     if is_interactive_mode:
         if args.color:
