@@ -1,10 +1,5 @@
-# -*- coding: utf-8 -*-
-
 # This module allows to query sqlite databases using RBQL
 
-from __future__ import unicode_literals
-from __future__ import print_function
-
 
 # TODO consider to support table names in "FROM" section of the query, making table_name param of SqliteRecordIterator optional
 # TODO consider adding support for multiple variable_prefixes i.e. "a" and <table_name> or "b" and <join_table_name> to alias input and join tables
